//  Demo 13: ThreadPracticeB.java
//
//   Another (More Complex) Java Application
//   Using the FirstMovingSquareThread Class.
//   
//   Illustrates creating separate Threads of execution by 
//   calling the run method of a Thread class we have written.
//
//   Graphically demonstrates separate Threads in action.
//
//             by Charles W. Neville
//      Copyright Charles W. Neville, July, 1999

// This application launches Squares which move across the screen.
// It contains TextFields to set the starting position, size and 
// Color of each Square, TextFields to set the x and y components
// of the motion direction vector, a TextField to set the distance
// for each Square, and a Button to launch a new Square.

// The event processing in this application is possible because each
// Square is animated by a separate Thread, thus leaving the main
// application free to process events and launch new Squares.
// When you run this application, you can create new Threads by
// clicking the Launch Button, and you can watch the separate 
// Threads running concurrently as the Squares move across the 
// screen.

// Unfortunately, running separate animation Threads produces alot
// of flicker, even with Graphics double buffering, because they
// all end up writing on the same Graphics object, namely the one
// for the frame.  In the next two demos, we will see how to  
// eliminate this flicker.

// Threads:
//
//   1. Main Thread -- this application.  Every running Java program
//        contains at least one Thread, the Thread main() runs on.
//   2. A separate Thread for the animation.  The moving Squares
//        are actually separate FirstMovingSquareThread objects,
//        launched by invoking the library start() method of the Thread
//        class. The start() method sets the new Threads up to run and 
//        then calls the run() method of our FirstMovingSquareThread 
//        class.  Our run() method is where the animation code goes.
      
import java.awt.*;
import java.awt.event.*;
import CloseAndExit;
import FirstMovingSquareThread;

public class ThreadPracticeB extends Frame 
                             implements ActionListener {

    FirstMovingSquareThread fmst;  // A single FirstMovingSquareThread 
                                   // object.  Each time we launch a 
                                   // new moving Square, we will 
                                   // reassign fmst to the new object.
 
    // for new Square objects
    Label xLabel, yLabel,    // Labels for x, y components of 
          sizeLabel,         // position, size and color
          colorLabel;

    TextField xInput,        // TextFields for same
              yInput,
              sizeInput,
              colorInput;

    // for new FirstMovingSquareThread objects 
    Label rxLabel, ryLabel,  // Labels for x, y components of motion 
          dLabel;            // vector, and distance

    TextField rxInput,       // TextFields for same
              ryInput,
              dInput; 

    Button launchButton;     // launch a new FirstMovingSquareThread

    int x, y, size;  // x and y components of new Square postion,
                     // plus size of new Square

    Color color;     // Color of new Square

    int rx, ry, d;   // x and y components of motion vector, plus
                     // distance, in units of the motion vector
                     // length

    // Constructor: builds Frame and initializes variables.
    public ThreadPracticeB() {

       setLayout(new FlowLayout());

       // for new Square objects
       xLabel = new Label(" x Coord ");
       xInput = new TextField(4);
       xInput.setText("50");
       add(xLabel);  add(xInput);

       yLabel = new Label(" y Coord ");
       yInput = new TextField(4);
       yInput.setText("100");
       add(yLabel);  add(yInput);

       sizeLabel = new Label(" Size ");
       sizeInput = new TextField(4);
       sizeInput.setText("25");
       add(sizeLabel);  add(sizeInput);

       colorLabel = new Label("Color");
       colorInput = new TextField(5);
       colorInput.setText("blue");
       add(colorLabel);  add(colorInput);

       // for new FirstMovingSquareThread objects
       rxLabel = new Label(" x Motion ");
       rxInput = new TextField(3);
       rxInput.setText("1");
       add(rxLabel);  add(rxInput);

       ryLabel = new Label(" y Motion ");
       ryInput = new TextField(3);
       ryInput.setText("1");
       add(ryLabel);  add(ryInput);

       dLabel = new Label(" Distance ");
       dInput = new TextField(4);
       dInput.setText("200");
       add(dLabel);  add(dInput);

       launchButton = new Button("Launch");
       launchButton.addActionListener(this);
       add(launchButton);

       x = 50;  y = 100; size = 25; color = Color.blue;
       rx = 1; ry = 1;   d = 200;
 
       addWindowListener(new CloseAndExit());   // as usual

    } // Constructor ThreadPracticeA()

    public void actionPerformed(ActionEvent evt) {

      String colorString;

      try {
        x = Integer.parseInt(xInput.getText());
      }
      catch (NumberFormatException e) {
        xInput.setText(Integer.toString(x)); // re-establish former
                                             // value
      }
 
      try {
        y = Integer.parseInt(yInput.getText());
      }
      catch (NumberFormatException e) {
        yInput.setText(Integer.toString(y)); // re-establish former
                                             // value
      }

      try {
        size = Integer.parseInt(sizeInput.getText());
      }
      catch (NumberFormatException e) {
        sizeInput.setText(Integer.toString(size)); 
                                      // re-establish former value
      }

      colorString = colorInput.getText();
      // always use the equals() method of the String class to
      // compare Strings for equality.  The == operator does object
      // comparison, that is it compares addresses, not content.
      if (colorString.equals("red"))
        color = Color.red;
      else if (colorString.equals("white"))
        color = Color.white;
      else if (colorString.equals("blue"))
        color = Color.blue;
      else if (colorString.equals("green"))
        color = Color.green;
      else { // restore default
        color = Color.blue;
        colorInput.setText("blue");
      }

      try {
        rx = Integer.parseInt(rxInput.getText());
      }
      catch (NumberFormatException e) {
        rxInput.setText(Integer.toString(rx)); // re-establish
                                               // former value
      }

      try {
        ry = Integer.parseInt(ryInput.getText());
      }
      catch (NumberFormatException e) {
        ryInput.setText(Integer.toString(ry)); // re-establish
                                               // former value
      }

      try {
        d = Integer.parseInt(dInput.getText());
      }
      catch (NumberFormatException e) {
        dInput.setText(Integer.toString(d)); // re-establish former
                                             // value
      }

      // create the fmst FirstMovingSquareThread object.
      fmst = new FirstMovingSquareThread( new Square(x, y, size, 
                    color), this, rx, ry, d);

      fmst.start();

    } // actionPerformed()

    public static void main(String[ ] args) {

       ThreadPracticeB tpb = new ThreadPracticeB();
       tpb.setSize(400, 400);
       tpb.setVisible(true);

    } // main()

} // class ThreadPracticeB
